import {
  geoNaturalEarth1,
  geoPath,
  geoGraticule,
  scaleQuantile,
  schemeYlGnBu,
} from 'd3';

const projection = geoNaturalEarth1();
const path = geoPath(projection);
const graticule = geoGraticule();

export const map = (
  selection,
  {
    allData,
    colorValue,
    missingDataFill = 'gray',
  }
) => {
  const { countries, data } = allData;

  const a3ToRow = new Map();
  for (const d of data) {
    a3ToRow.set(d.country, d);
  }

  const colorScale = scaleQuantile()
    .domain(data.map(colorValue))
    .range(schemeYlGnBu[9].reverse());

  selection
    .selectAll('path.graticule')
    .data([null])
    .join('path')
    .attr('class', 'graticule')
    .attr('d', path(graticule()))
    .attr('fill', 'none')
    .attr('stroke', '#BBB')
    .attr('stroke-width', 0.5);

  selection
    .selectAll('path.outline')
    .data([null])
    .join('path')
    .attr('class', 'outline')
    .attr('d', path(graticule.outline()))
    .attr('fill', 'none')
    .attr('stroke', 'black')
    .attr('stroke-width', 2);

  selection
    .selectAll('path.country')
    .data(countries.features)
    .join('path')
    .attr('d', path)
    .attr('class', 'country')
    .attr('stroke', '#AAA')
    .attr('fill', (feature) => {
      const d = a3ToRow.get(
        feature.properties.a3
      );
      return d && colorValue(d) !== null
        ? colorScale(colorValue(d))
        : missingDataFill;
    })
    .attr('stroke-width', 0.5);
};
