import { csvParse, select } from 'd3';
import topojson from 'topojson-client';
import { map } from './map';

const worldAtlasURL =
  'https://unpkg.com/visionscarto-world-atlas@0.1.0/world/110m.json';

const dataURL =
  'https://gist.githubusercontent.com/curran/4914da157af89894e4190b21452a54d3/raw/a10ed7ebaf4689fe8be20f47dfc9ffc13bfdda8b/API_NY.GDP.PCAP.PP.CD_DS2_en_csv_v2_4901661.csv';

export const viz = (
  container,
  { state, setState }
) => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An object containing `countries` and `data`
  const { allData } = state;

  if (allData && allData !== 'LOADING') {
    svg.call(map, {
      allData,
      colorValue: (d) => d.gdp,
    });
  }

  if (allData === undefined) {
    setState((state) => ({
      ...state,
      allData: 'LOADING',
    }));
    Promise.all([
      fetch(worldAtlasURL)
        .then((response) => response.json())
        .then((topoJSONData) =>
          topojson.feature(
            topoJSONData,
            'countries'
          )
        ),
      fetch(dataURL)
        .then((response) => response.text())
        .then((csvString) => {
          const validCSV = csvString.substring(
            86
          );
          return csvParse(validCSV).map((d) => ({
            gdp:
              d['2020'] === ''
                ? null
                : +d['2020'],
            country: d['Country Code'],
          }));
        }),
    ]).then(([countries, data]) => {
      setState((state) => ({
        ...state,
        allData: { countries, data },
      }));
    });
  }
};
